/**
 *  @file       MelloBell.ck
 *
 *  @author     Colin Sullivan <colinsul [at] gmail.com>
 *
 *              Copyright (c) 2012 Colin Sullivan
 *              Licensed under the MIT license.
 **/

/**
 *  @class      A nice mellow percussive bell-like sound
 *  @extends    Instrument
 **/
public class MelloBell extends Instrument {
    Envelope e => blackhole;

    SinOsc carrier => outputsAll;
    carrier.gain(0);


    // FM
    carrier.sync(2);
    SinOsc modulator => carrier;
    
    3 => float modulatorIndex;
    10 => float modulatorIndex2;

    fun void noteOn(float onVelocity) {
        1+(onVelocity*2) => modulatorIndex;
        2+(onVelocity*8) => modulatorIndex2;
        e.duration(0.01::second);
        e.keyOn();
    }

    fun void noteOff() {
        e.keyOff();
    }

    float modulatorFreq;
    float carrierFreq;
    fun float freq(float aFreq) {
        aFreq => _freq;

        aFreq => carrierFreq;
        aFreq/2 => modulatorFreq;
        modulator.freq(modulatorFreq);
        carrier.freq(carrierFreq);

        return this.freq();
    }

    fun void envelopeHandler() {
        float expValue;

        while(true) {
            Math.pow(e.value(), 3.0) => expValue;

            // modulator envelope
            expValue*((modulatorIndex2 - modulatorIndex)*modulatorFreq) + modulatorIndex*modulatorFreq => modulator.gain;


            // Carrier envelope
            expValue => carrier.gain;

            1::ms => now;
        }
    }
    spork ~ envelopeHandler();

    fun void parameterTester() {
        while(true) {
            // Increasing modulator index sounds really cool
            // 0.01 +=> modulatorIndex;
            // modulator.gain(modulatorFreq*modulatorIndex);

            // 0.1 +=> modulatorFreq;
            // modulator.freq(modulatorFreq);
            // modulator.gain(modulatorFreq*modulatorIndex);

            10::ms => now;
        }
    }
    // spork ~ parameterTester();

    fun void playNote(float onVelocity) {
        this.noteOn(onVelocity);
        0.01::second => now;
        e.duration(1.75::second);
        this.noteOff();
        2::second => now;        
    }

}

// MelloBell m;
// m.freq(440);
// spork ~ m.playNote(1.0);
// 1::second => now;
