/**
 *  @file       RandomBellsPerformer.ck
 *
 *  @author     Colin Sullivan <colinsul [at] gmail.com>
 *
 *              Copyright (c) 2012 Colin Sullivan
 *              Licensed under the MIT license.
 **/

/**
 *  @class      Play a bunch of `MellowBell` instances at psuedo-random
 *              times and pitches.
 *  @extends    Performer
 **/
public class RandomBellsPerformer extends Performer {

    MelloBell bells[8];
    for(0 => int i; i < bells.size(); i++) {
        bells[i].gain(0.3);
    }

    // Augmented Scale
    // C    Eb  E   G   G#  B   C
    // 60   63  64  67  68  71  72
    // A
    // 57   60  61  64  65  68  69
    // [57, 60, 61, 64, 65, 68, 69] @=> int bellNotes[];

    // Tritone Scale
    // C    Db  E   Gb  G   Bb  C
    // 60   61  64  66  67  70  72
    // A
    // 57   58  61  63  64  67  69
    [57, 58, 61, 63, 64, 67, 69] @=> int bellNotes[];
    dur bellDurs[40];
    0 => int numBellDurs;
    1.5::second => bellDurs[numBellDurs];
    1 +=> numBellDurs;

    0.10 => float bellBaseVelocity;

    false => int _playing;

    fun void play() {
        this.pre_play();

        true => _playing;


        while(_playing) {
            for(0 => int i; i < bells.size(); i++) {
                bellNotes[Std.rand2(0, bellNotes.size()-1)] => int note;
                Std.rand2(-1, 1) => int octave;
                bells[i].freq(Std.mtof(note+(octave*12)));
                spork ~ bells[i].playNote(Std.rand2f(0.75, 1.0)*bellBaseVelocity);
                bellDurs[Std.rand2(0, numBellDurs-1)] => now;
            }
        }
    }

    fun void stop() {
        false => _playing;
    }

    fun void speedUpNotes(dur totalDuration) {
        totalDuration/3 => dur waitTime;

        for(0 => int i; i < 3; i++) {
            bellDurs[numBellDurs-1]/2 => bellDurs[numBellDurs];
            1 +=> numBellDurs;                    

            bellDurs[numBellDurs-1]/2 => bellDurs[numBellDurs];
            1 +=> numBellDurs;                    

            if(i == 0 || i == 1) {
                // Remove oldest duration
                for(1 => int j; j < numBellDurs-1; j++) {
                    bellDurs[j] => bellDurs[j-1];
                }
                1 -=> numBellDurs;                    
            }

            0.15 +=> bellBaseVelocity;

            waitTime => now;


            // waitTime-1::second => waitTime;
        }
    }

    fun void slowDownNotes(dur totalDuration) {
        totalDuration/numBellDurs => dur waitTime;
        while(true) {
            for(0 => int i; i < numBellDurs; i++) {
                bellDurs[i]*2 => bellDurs[i];

                // 2::second => now;
                // waitTime-1::second => waitTime;
                waitTime => now;
            }

        }
    }

}