/**
 *  @file       SeparatingFM.ck
 *
 *  @author     Colin Sullivan <colinsul [at] gmail.com>
 *
 *              Copyright (c) 2012 Colin Sullivan
 *              Licensed under the MIT license.
 **/


/**
 *  @class      This basic tone sounds like it begins to "separate".
 *  @extends    Instrument
 **/
public class SeparatingFM extends Instrument {
    Envelope e => blackhole;
    e.duration(0.01::second);

    SinOsc carrier => outputsAll;
    carrier.gain(0);

    // A3 - 220Hz
    57 => Std.mtof => float carrierFreq;
    carrier.freq(carrierFreq);

    // FM
    carrier.sync(2);
    SinOsc modulator => carrier;

    float _modulatorIndex2;
    float _modulatorIndex;
    float _modulatorFreq;
    fun float modulatorIndex(float aIndex) {
        aIndex => _modulatorIndex;
        modulator.gain(((_modulatorIndex2 - _modulatorIndex)*_modulatorFreq) + _modulatorIndex*_modulatorFreq);
        return this.modulatorIndex();
    }
    fun float modulatorIndex() {
        return _modulatorIndex;
    }
    this.modulatorIndex(1);

    fun float modulatorIndex2(float aIndex) {
        if(aIndex > 90) {
            return this.modulatorIndex2();
        }
        aIndex => _modulatorIndex2;
        modulator.gain(((_modulatorIndex2 - _modulatorIndex)*_modulatorFreq) + _modulatorIndex*_modulatorFreq);
        return this.modulatorIndex2();
    }
    fun float modulatorIndex2() {
        return _modulatorIndex2;
    }
    this.modulatorIndex2(5);

    fun float modulatorFreq() {
        return _modulatorFreq;
    }
    fun float modulatorFreq(float aFreq) {
        aFreq => _modulatorFreq;
        modulator.freq(_modulatorFreq);
        modulator.gain(((_modulatorIndex2 - _modulatorIndex)*_modulatorFreq) + _modulatorIndex*_modulatorFreq);
        return this.modulatorFreq();
    }
    // A1 - 55Hz
    this.modulatorFreq(Std.mtof(33));


    fun void envelopeHandler() {
        float expValue;

        while(true) {
            Math.pow(e.value(), 3.0) => expValue;

            // modulator envelope
            expValue*((_modulatorIndex2 - _modulatorIndex)*_modulatorFreq) + _modulatorIndex*_modulatorFreq => modulator.gain;


            // Carrier envelope
            expValue => carrier.gain;

            1::ms => now;
        }
    }
    spork ~ envelopeHandler();

    fun void increaseModulatorFreq(dur increaseDuration) {
        Std.mtof(51) => float destFreq;

        10::ms => dur increaseInterval;

        increaseDuration/increaseInterval => float numIncreases;
        (destFreq - _modulatorFreq)/numIncreases => float increaseAmt;

        while(this.modulatorFreq() < destFreq) {
            // Increasing modulator index sounds really cool
            // 0.01 +=> _modulatorIndex;
            // modulator.gain(_modulatorFreq*_modulatorIndex);

            increaseAmt+this.modulatorFreq() => this.modulatorFreq;

            increaseInterval => now;
        }
    }

    fun void decreaseModulatorFreq(dur decreaseDuration) {
        Std.mtof(33) => float destFreq;

        10::ms => dur decreaseInterval;

        decreaseDuration/decreaseInterval => float numDecreases;
        (this.modulatorFreq() - destFreq)/numDecreases => float decreaseAmt;

        while(this.modulatorFreq() > destFreq) {
            // Increasing modulator index sounds really cool
            // 0.01 +=> _modulatorIndex;
            // modulator.gain(_modulatorFreq*_modulatorIndex);

            this.modulatorFreq()-decreaseAmt => this.modulatorFreq;

            decreaseInterval => now;
        }        
    }

    fun void increaseModulatorIndex2(dur duration) {
        90 => float goalIndex;
        duration/0.5::ms => float numIncreases;

        (goalIndex - this.modulatorIndex2())/numIncreases => float increaseAmt;
        
        now+duration => time end;
        while(now < end && this.modulatorIndex2() < goalIndex) {
            this.modulatorIndex2(increaseAmt+this.modulatorIndex2());
            0.5::ms => now;
        }
    }

    fun void decreaseModulatorIndex2(dur duration) {
        5 => float goalIndex;
        duration/0.5::ms => float numDecreases;

        (this.modulatorIndex2() - goalIndex)/numDecreases => float decreaseAmt;
        now+duration => time end;
        while(now < end && this.modulatorIndex2() > goalIndex) {
            this.modulatorIndex2(this.modulatorIndex2()-decreaseAmt);
            0.5::ms => now;
        }
    }

    fun void noteOn() {
        e.keyOn();
    }

    fun void noteOn(dur aDuration) {
        e.duration(aDuration);
        e.keyOn();
    }

    fun void noteOff(dur aDuration) {
        e.duration(aDuration);
        e.keyOff();
    }

    fun void playNote() {
        this.noteOn();
        _duration => now;
        this.noteOff();
    }

    fun void playTest() {
        // this.duration(26::second);
        this.modulatorFreq(Std.mtof(51));
        this.noteOn();
        2::second => now;
        // spork ~ decreaseModulatorFreq(18::second);
        spork ~ increaseModulatorIndex2(10::second);
        18::second => now;
    }
}

// SeparatingFM test;
// test.playTest();

