/**
 *  @file       UFOEngine.ck
 *
 *  @author     Colin Sullivan <colinsul [at] gmail.com>
 *
 *              Copyright (c) 2012 Colin Sullivan
 *              Licensed under the MIT license.
 **/

/**
 *  @class      A time-varying engine-like pulsing synth sound.
 *  @extends    Instrument
 **/
public class UFOEngine extends Instrument {
    SinOsc carrier => Envelope e => LPF f => outputsAll;
    SawOsc sawCarrier => e => outputsAll;

    // FM
    2 => sawCarrier.sync => carrier.sync;
    SinOsc modulator => carrier;
    modulator => sawCarrier;

    SinOsc filterModulator => blackhole;
    filterModulator.gain(4500);
    f.Q(5);

    float _filterModulatorFreq;
    fun float filterModulatorFreq(float aFreq) {
        aFreq => _filterModulatorFreq;
        filterModulator.freq(_filterModulatorFreq);

        return filterModulatorFreq();
    }
    fun float filterModulatorFreq() {
        return _filterModulatorFreq;
    }
    filterModulatorFreq(0.5);

    fun void modulate_filter() {

        while(true) {
            // Filter frequency oscillates between 10000 and 1000
            10000 - (4500 + filterModulator.last()) => f.freq;
            0.01::second => now;
        }
    }
    spork ~ modulate_filter();

    fun void envelopeToFilterModulatorFreq() {
        while(true) {
            e.value()*_filterModulatorFreq => filterModulator.freq;

            1::ms => now;
        }
    }


    fun float freq(float aFreq) {
        aFreq => _freq;

        _freq => carrier.freq;

        return this.freq();
    }
    Std.mtof(60) => this.freq;

    sawCarrier.gain(0.10);
    carrier.freq()/2 => sawCarrier.freq;

    modulator.gain(400);
    modulator.freq(
        Std.mtof(20)
    );

    fun void noteOn() {
        e.keyOn();
    }

    fun void noteOn(dur aDuration) {
        e.duration(aDuration);
        e.keyOn();
    }

    fun void noteOff() {
        e.keyOff();
    }

    fun void noteOff(dur aDuration) {
        e.duration(aDuration);
        e.keyOff();
    }

    fun void playTest() {
        this.noteOn();
        9::second => now;
        this.noteOff();
        1::second => now;        
    }
}

// UFOEngine test;
// test.playTest();
